package edu.mit.appinventor;

import com.google.appinventor.components.annotations.DesignerComponent;
import com.google.appinventor.components.annotations.PropertyCategory;
import com.google.appinventor.components.annotations.SimpleEvent;
import com.google.appinventor.components.annotations.SimpleFunction;
import com.google.appinventor.components.annotations.SimpleObject;
import com.google.appinventor.components.annotations.SimpleProperty;
import com.google.appinventor.components.common.ComponentCategory;
import com.google.appinventor.components.runtime.AndroidNonvisibleComponent;
import com.google.appinventor.components.runtime.Canvas;
import com.google.appinventor.components.runtime.ComponentContainer;
import com.google.appinventor.components.runtime.EventDispatcher;

import android.content.Context;
import android.view.MotionEvent;
import android.util.Log;


@DesignerComponent(version = 1,
description = "A ...",
category = ComponentCategory.EXTENSION,
nonVisible = true,
iconName = "images/extension.png")
@SimpleObject(external = true)
public class MyRotationDetector extends AndroidNonvisibleComponent {
	private Canvas myCanvas;
	
	public MyRotationDetector(ComponentContainer container) {
		super(container.$form());
	}
	@SimpleFunction
	  public void AddHandlerToCanvas(Canvas myCanvas) {
	    this.myCanvas = myCanvas;
	    // The key step here is to create the gesture detector and register it for the Canvas
	    // the gesture detector is created from a gesture listener.  The activity context for detection
	    // the will be the context of the canvas.
	    ExtensionRotationDetector myDetector = 
	        new ExtensionRotationDetector(myCanvas.getContext(), new MyOnRotationGestureListener());
	    myCanvas.registerCustomGestureDetector(myDetector); 
	  }

	  @SimpleProperty(category = PropertyCategory.BEHAVIOR)
	  public Canvas MyCanvas() {
	    return myCanvas;
	  }
	  
	 @SimpleEvent
	  public void Rotate(double angle) {
	    EventDispatcher.dispatchEvent(this, "Rotate", angle);
	  }
	  
	 // The gesture detector to be added should extend an ordinary Android gesture detector, and
	 // also implement the Canvas.ExtensioGestureDetector interface
	 // It's created from a gesture listener and an activity context
	 public class ExtensionRotationDetector extends RotationDetector 
	         implements Canvas.ExtensionGestureDetector {
	   public ExtensionRotationDetector(Context c, OnRotationGestureListener l ) {
	     super(c,l);
	   }
	 }

	 // Here's the gesture listener. 
	 
	 public class MyOnRotationGestureListener implements
	            OnRotationGestureListener {
		 @Override
		 public void OnRotation(RotationDetector rotationDetector) {
			 // TODO Auto-generated method stub
			 float angle = rotationDetector.getAngle();
			 Rotate((double) angle);
		 }
	 }

	 public class RotationDetector {
	 	protected final Context mContext;
	 	private static final int INVALID_POINTER_ID = -1;
	     private float fX, fY, sX, sY,oldval,newval;
	     private int ptrID1, ptrID2;
	     private float mAngle;

	     private OnRotationGestureListener mListener;

	     public float getAngle() {
	         return mAngle;
	     }

	     public RotationDetector(Context c, OnRotationGestureListener listener){
	     	mContext = c;
	         mListener = listener;
	         ptrID1 = INVALID_POINTER_ID;
	         ptrID2 = INVALID_POINTER_ID;
	     }

	     public boolean onTouchEvent(MotionEvent event){
	         switch (event.getActionMasked()) {
	             case MotionEvent.ACTION_DOWN:
	                 ptrID1 = event.getPointerId(event.getActionIndex());
	                 break;
	             case MotionEvent.ACTION_POINTER_DOWN:
	                 ptrID2 = event.getPointerId(event.getActionIndex());
	                 sX = event.getX(event.findPointerIndex(ptrID1));
	                 sY = event.getY(event.findPointerIndex(ptrID1));
	                 fX = event.getX(event.findPointerIndex(ptrID2));
	                 fY = event.getY(event.findPointerIndex(ptrID2));
	                 oldval = sX+sY+fY+fX;
	                 break;
	             case MotionEvent.ACTION_MOVE:
	            	 Log.i("Rotate","Action Move");
	                 if(ptrID1 != INVALID_POINTER_ID && ptrID2 != INVALID_POINTER_ID){
	                     float nfX, nfY, nsX, nsY;
	                     nsX = event.getX(event.findPointerIndex(ptrID1));
	                     nsY = event.getY(event.findPointerIndex(ptrID1));
	                     nfX = event.getX(event.findPointerIndex(ptrID2));
	                     nfY = event.getY(event.findPointerIndex(ptrID2));
	                     Log.i("Rotate", String.format("First point:(%f,%f), Second Point(%f,%f)", nsX,nsY,nfX,nfY));
	                     newval = nsX+nsY+nfX+nfY;
	                     if(Math.abs(newval-oldval)>5){
	                    	 Log.i("Rotate", "Fire Rotation");
	                    	 mAngle = angleBetweenLines(fX, fY, sX, sY, nfX, nfY, nsX, nsY);

	                    	 if (mListener != null) {
	                    		 mListener.OnRotation(this);
	                    	 }
	                    	 oldval = newval;
	                     }
	                 }
	                 break;
	             case MotionEvent.ACTION_UP:
	                 ptrID1 = INVALID_POINTER_ID;
	                 break;
	             case MotionEvent.ACTION_POINTER_UP:
	                 ptrID2 = INVALID_POINTER_ID;
	                 break;
	             case MotionEvent.ACTION_CANCEL:
	                 ptrID1 = INVALID_POINTER_ID;
	                 ptrID2 = INVALID_POINTER_ID;
	                 break;
	         }
	         return true;
	     }

	     private float angleBetweenLines (float fX, float fY, float sX, float sY, float nfX, float nfY, float nsX, float nsY)
	     {
	    	 float v1x = fX-sX;
	    	 float v1y = -(fY-sY);
	    	 float v2x = nfX-nsX;
	    	 float v2y = -(nfY-nsY);
	         float angle1 = normalizeAngle((float) Math.atan2(v1y, v1x),v1x,v1y);
	         float angle2 = normalizeAngle((float) Math.atan2(v2y, v2x),v2x,v2y);
	         Log.i("Rotate", String.format("Vector1 (%f,%f) Vector2 (%f,%f)", v1x,v1y,v2x,v2y));
	         float angle = (float)Math.toDegrees(angle1-angle2);
	         return angle;
	     }
	     private float normalizeAngle(float angle,float x, float y){
	    	 if(x>=0&&y>=0){
	    		 return angle;
	    	 }else if(x>=0&&y<0){
	    		 return angle;
	    	 }else if(x<0&&y>=0){
	    		 return (float)Math.PI-angle;
	    	 }else{
	    		 return -(float)Math.PI+angle;
	    	 }
	     }
	 }
	 public static interface OnRotationGestureListener {
         public void OnRotation(RotationDetector rotationDetector);
     }
	 
}
